#include "stdafx.h"
#include "HookDllInterface.h"
#include "OSInfo.h"

const static LPCTSTR g_szPwdSpyHkDll = _T("PwdSpyHk.dll");

#if defined(UNICODE) || defined(_UNICODE)

#ifdef _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKUD
#else	// _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKUR
#endif	// _DEBUG

#else	// UNICODE

#ifdef _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKD
#else	// _DEBUG
#define HOOKDLL_ID	IDR_PWDSPYHKR
#endif	// _DEBUG

#endif	// UNICODE

//***********************************************
CHookDllInterface::CHookDllInterface()
	: m_hModHookDll(NULL), m_lpfnInstallHook(NULL), m_lpfnRemoveHook(NULL), m_lpfnScanPassword(NULL)
{
	COSInfo osi;

	if(osi.IsNT() && osi.GetMajor() >= 5)
	{
		ExtractResource(AfxGetInstanceHandle(), HOOKDLL_ID, g_szPwdSpyHkDll);

		m_hModHookDll = LoadLibrary(g_szPwdSpyHkDll);
		if(m_hModHookDll)
		{
			m_lpfnInstallHook = (LPFNINSTALLHOOK)GetProcAddress(m_hModHookDll, "InstallHook"); _ASSERTE(m_lpfnInstallHook);
			m_lpfnRemoveHook = (LPFNREMOVEHOOK)GetProcAddress(m_hModHookDll, "RemoveHook"); _ASSERTE(m_lpfnRemoveHook);
			m_lpfnScanPassword = (LPFNSCANPASSWORD)GetProcAddress(m_hModHookDll, "ScanPassword"); _ASSERTE(m_lpfnScanPassword);
		}
	}
}

//***********************************************
CHookDllInterface::~CHookDllInterface()
{
	// Make absolutely sure we unhook before closing
	if(m_lpfnRemoveHook)
		m_lpfnRemoveHook();

	if(m_hModHookDll)
	{
		FreeLibrary(m_hModHookDll);
		DeleteFile(g_szPwdSpyHkDll);
	}
}

//***********************************************
bool CHookDllInterface::ExtractResource(const HINSTANCE hInstance, const WORD nID, LPCTSTR szFilename)
{
	_ASSERTE(hInstance);
	_ASSERTE(nID);
	_ASSERTE(szFilename);
	_ASSERTE(lstrlen(szFilename));

	bool bSuccess = false;

	try
	{
		HRSRC hResource = FindResource(hInstance, MAKEINTRESOURCE(nID), _T("BINARY")); _ASSERTE(hResource);
		HGLOBAL hFileResource = LoadResource(hInstance, hResource); _ASSERTE(hFileResource);
		LPVOID lpFile = LockResource(hFileResource);

		DWORD dwSize = SizeofResource(hInstance, hResource);

		// Open the file and filemap
		HANDLE hFile = CreateFile(szFilename, GENERIC_READ | GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
		HANDLE hFilemap = CreateFileMapping(hFile, NULL, PAGE_READWRITE, 0, dwSize, NULL);

		// Get a pointer to write to
		LPVOID lpBaseAddress = MapViewOfFile(hFilemap, FILE_MAP_WRITE, 0, 0, 0);

		// Write the file
		CopyMemory(lpBaseAddress, lpFile, dwSize);

		// Unmap the file and close the handles
		UnmapViewOfFile(lpBaseAddress);
		CloseHandle(hFilemap);
		CloseHandle(hFile);
	}
	catch(...) {}

	return bSuccess;
}

//***********************************************
bool CHookDllInterface::InstallHook(const DWORD dwThreadId)
{
	return (m_lpfnInstallHook == NULL) ? false : m_lpfnInstallHook(dwThreadId);
}

//***********************************************
bool CHookDllInterface::RemoveHook(void)
{
	return (m_lpfnRemoveHook == NULL) ? false : m_lpfnRemoveHook();
}

//***********************************************
bool CHookDllInterface::ScanPassword(const HWND hWnd, const HWND hPwdSpyWnd)
{
	return (m_lpfnScanPassword == NULL) ? false : m_lpfnScanPassword(hWnd, hPwdSpyWnd);
}
